/*$Id: acre_vector_broadcast.c,v 1.1 2004/04/30 Joel M. Malard Exp d3j540 $ */

#include "tao_solver.h"
#include "petscblaslapack.h"
#include "petsclog.h"
#include "acre.h"
#include "petsclog.h"
#include <assert.h>
#include <mpi.h>
#include <math.h>
#include "copyright.h"
#undef __FUNC__  
#define __FUNC__ "acre_vector_broadcast"
int acre_vector_broadcast(acre_cntxt *cntxt, Vec u, int root )
{
	int start, end, rank;
	int npes = cntxt->qc;
	int i, ierr, *rows;
	PetscScalar *vals;
	MPI_Comm comm = cntxt->rcomm;

	HELLO("acre_vector_broadcast enter");

	ierr = MPI_Comm_size(comm, &npes);
		assert(ierr==MPI_SUCCESS);

	ierr = MPI_Comm_rank(comm, &rank);
		assert(ierr==MPI_SUCCESS);
	
	if ( npes==1 ) return 0;

	ierr = VecGetOwnershipRange(u, &start, &end);	CHKERRQ(ierr);

	if ( rank==root )
	{
		ierr = VecGetArray(u, &vals);		CHKERRQ(ierr);
		ierr = MPI_Bcast(vals, end-start,
			MPI_DOUBLE, root, comm);
			assert(ierr==MPI_SUCCESS);
		ierr = VecRestoreArray(u, &vals);	CHKERRQ(ierr);
	}
	else
	{
		ierr = PetscMalloc((end-start)*sizeof(
			PetscScalar), &vals);		CHKERRQ(ierr);
		ierr = PetscMalloc((end-start)*sizeof(int)
			,&rows);			CHKERRQ(ierr);
		for ( i=start; i<end; i++ )
			rows[i-start] = i;

		ierr = MPI_Bcast(vals, end-start,
			MPI_DOUBLE, root, comm);
			assert(ierr==MPI_SUCCESS);

		ierr = VecSetValues( u, end-start,
			rows, vals, INSERT_VALUES);	CHKERRQ(ierr);
		ierr = VecAssemblyBegin(u);		CHKERRQ(ierr);
		ierr = VecAssemblyEnd(u);		CHKERRQ(ierr);

		PetscFree(rows);
		PetscFree(vals);
	}

	HELLO("acre_vector_broadcast exit");

	return 0;
}
